@@ -67,7 +67,7 @@ void my_net_write_callback(int command)
 void print_usage(void)
 {
 	printf("mosquitto_pub is a simple mqtt client that will publish a message on a single topic and exit.\n\n");
-	printf("Usage: mosquitto_pub [-h host] [-i id] [-m message] [-p port] [-q qos] [-r] [-t topic]\n\n");
+	printf("Usage: mosquitto_pub [-h host] [-i id] [-p port] [-q qos] [-r] -m message -t topic\n\n");
 	printf(" -h : mqtt host to connect to. Defaults to localhost.\n");
 	printf(" -i : id to use for this client. Defaults to mosquitto_pub_ appended with the process id.\n");
 	printf(" -m : message payload to send.\n");
