@@ -5,23 +5,27 @@
 #include <config.h>
 #include <mqtt3.h>
 
+int _mqtt3_conf_parse_bool(char **token, const char *name, bool *value);
+int _mqtt3_conf_parse_int(char **token, const char *name, int *value);
+
 void mqtt3_config_init(mqtt3_config *config)
 {
 	/* Set defaults */
-	config->daemon = 0;
+	config->autosave_interval = 1800;
+	config->daemon = false;
 #ifdef __CYGWIN__
 	config->ext_sqlite_regex = "./sqlite3-pcre.dll";
 #else
-	config->ext_sqlite_regex = "./sqlite3-pcre.so";
+	config->ext_sqlite_regex = "/usr/lib/sqlite3/pcre.so";
 #endif
 	config->iface = NULL;
 	config->iface_count = 0;
 	config->log_dest = MQTT3_LOG_STDERR;
 	config->log_type = MQTT3_LOG_ERR | MQTT3_LOG_WARNING | MQTT3_LOG_NOTICE | MQTT3_LOG_INFO;
-	config->msg_timeout = 10;
-	config->persistence = 0;
+	config->persistence = false;
 	config->persistence_location = NULL;
 	config->pid_file = NULL;
+	config->retry_interval = 20;
 	config->sys_interval = 10;
 	config->user = "mosquitto";
 }
@@ -45,7 +49,7 @@ int mqtt3_config_parse_args(mqtt3_config *config, int argc, char *argv[])
 			}
 			i++;
 		}else if(!strcmp(argv[i], "-d") || !strcmp(argv[i], "--daemon")){
-			config->daemon = 1;
+			config->daemon = true;
 		}else if(!strcmp(argv[i], "-i") || !strcmp(argv[i], "--interface")){
 			if(i<argc-1){
 				i++;
@@ -128,8 +132,10 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 			}
 			token = strtok(buf, " ");
 			if(token){
-
-				if(!strcmp(token, "ext_sqlite_regex")){
+				if(!strcmp(token, "autosave_interval")){
+					if(_mqtt3_conf_parse_int(&token, "autosave_interval", &config->autosave_interval)) return 1;
+					if(config->autosave_interval < 0) config->autosave_interval = 0;
+				}else if(!strcmp(token, "ext_sqlite_regex")){
 					token = strtok(NULL, " ");
 					if(token){
 						config->ext_sqlite_regex = mqtt3_strdup(token);
@@ -210,30 +216,8 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 					}else{
 						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty log_type value in configuration.");
 					}
-				}else if(!strcmp(token, "msg_timeout")){
-					token = strtok(NULL, " ");
-					if(token){
-						config->msg_timeout = atoi(token);
-						if(config->msg_timeout < 1 || config->msg_timeout > 3600){
-							mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid msg_timeout value (%d).", config->msg_timeout);
-							return 1;
-						}
-					}else{
-						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty msg_timeout value in configuration.");
-						return 1;
-					}
 				}else if(!strcmp(token, "persistence")){
-					token = strtok(NULL, " ");
-					if(token){
-						config->persistence = atoi(token);
-						if(config->persistence != 1 && config->persistence != 0){
-							mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid persistence value (%d).", config->persistence);
-							return 1;
-						}
-					}else{
-						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty persistence value in configuration.");
-						return 1;
-					}
+					if(_mqtt3_conf_parse_bool(&token, "persistence", &config->persistence)) return 1;
 				}else if(!strcmp(token, "persistence_location")){
 					token = strtok(NULL, " ");
 					if(token){
@@ -248,35 +232,29 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 						config->pid_file = mqtt3_strdup(token);
 					}
 				}else if(!strcmp(token, "port")){
-					token = strtok(NULL, " ");
-					if(token){
-						port_tmp = atoi(token);
-						if(port_tmp < 1 || port_tmp > 65535){
-							mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid port value (%d).", port_tmp);
-							return 1;
-						}
-						config->iface_count++;
-						config->iface = mqtt3_realloc(config->iface, sizeof(struct mqtt3_iface)*config->iface_count);
-						if(!config->iface){
-							mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Out of memory.");
-							return 1;
-						}
-						config->iface[config->iface_count-1].iface = NULL;
-						config->iface[config->iface_count-1].port = port_tmp;
-					}else{
-						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty port value in configuration.");
+					if(_mqtt3_conf_parse_int(&token, "port", &port_tmp)) return 1;
+					if(port_tmp < 1 || port_tmp > 65535){
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid port value (%d).", port_tmp);
+						return 1;
+					}
+					config->iface_count++;
+					config->iface = mqtt3_realloc(config->iface, sizeof(struct mqtt3_iface)*config->iface_count);
+					if(!config->iface){
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Out of memory.");
+						return 1;
+					}
+					config->iface[config->iface_count-1].iface = NULL;
+					config->iface[config->iface_count-1].port = port_tmp;
+				}else if(!strcmp(token, "retry_interval")){
+					if(_mqtt3_conf_parse_int(&token, "retry_interval", &config->retry_interval)) return 1;
+					if(config->retry_interval < 1 || config->retry_interval > 3600){
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid retry_interval value (%d).", config->retry_interval);
 						return 1;
 					}
 				}else if(!strcmp(token, "sys_interval")){
-					token = strtok(NULL, " ");
-					if(token){
-						config->sys_interval = atoi(token);
-						if(config->sys_interval < 1 || config->sys_interval > 65535){
-							mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid sys_interval value (%d).", config->sys_interval);
-							return 1;
-						}
-					}else{
-						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty sys_interval value in configuration.");
+					if(_mqtt3_conf_parse_int(&token, "sys_interval", &config->sys_interval)) return 1;
+					if(config->sys_interval < 1 || config->sys_interval > 65535){
+						mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid sys_interval value (%d).", config->sys_interval);
 						return 1;
 					}
 				}else if(!strcmp(token, "user")){
@@ -305,3 +283,35 @@ int mqtt3_config_read(mqtt3_config *config, const char *filename)
 
 	return rc;
 }
+
+int _mqtt3_conf_parse_bool(char **token, const char *name, bool *value)
+{
+	*token = strtok(NULL, " ");
+	if(*token){
+		if(!strcmp(*token, "false") || !strcmp(*token, "0")){
+			*value = false;
+		}else if(!strcmp(*token, "true") || !strcmp(*token, "1")){
+			*value = true;
+		}else{
+			mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Invalid %s value (%s).", name, *token);
+		}
+	}else{
+		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty %s value in configuration.", name);
+		return 1;
+	}
+	
+	return 0;
+}
+
+int _mqtt3_conf_parse_int(char **token, const char *name, int *value)
+{
+	*token = strtok(NULL, " ");
+	if(*token){
+		*value = atoi(*token);
+	}else{
+		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: Empty %s value in configuration.", name);
+		return 1;
+	}
+
+	return 0;
+}
