@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009, Roger Light <roger@atchoo.org>
+Copyright (c) 2009,2010, Roger Light <roger@atchoo.org>
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <errno.h>
 #include <fcntl.h>
 #include <ifaddrs.h>
+#include <netdb.h>
 #include <netinet/in.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -51,6 +52,10 @@ static uint64_t bytes_sent = 0;
 static unsigned long msgs_received = 0;
 static unsigned long msgs_sent = 0;
 
+#ifdef WITH_CLIENT
+void (*client_net_write_callback)(int) = NULL;
+#endif
+
 int _mqtt3_socket_listen(struct sockaddr *addr);
 
 int mqtt3_socket_accept(mqtt3_context **contexts, int context_count, int listensock)
@@ -128,26 +133,44 @@ int mqtt3_socket_close(mqtt3_context *context)
  * Returns -1 on failure (ip is NULL, socket creation/connection error)
  * Returns sock number on success.
  */
-int mqtt3_socket_connect(const char *ip, uint16_t port)
+int mqtt3_socket_connect(const char *host, uint16_t port)
 {
 	int sock;
-	struct sockaddr_in addr;
+	int opt;
+	struct addrinfo hints;
+	struct addrinfo *ainfo, *rp;
+	int s;
 
-	if(!ip) return -1;
+	if(!host || !port) return -1;
 
-	sock = socket(AF_INET, SOCK_STREAM, 0);
-	if(sock == -1){
+	memset(&hints, 0, sizeof(struct addrinfo));
+	hints.ai_family = AF_INET; /* IPv4 only at the moment. */
+	hints.ai_socktype = SOCK_STREAM;
+
+	s = getaddrinfo(host, NULL, &hints, &ainfo);
+	if(s) return -1;
+
+	for(rp = ainfo; rp != NULL; rp = rp->ai_next){
+		sock = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
+		if(sock == -1) continue;
+		
+		((struct sockaddr_in *)rp->ai_addr)->sin_port = htons(port);
+		if(connect(sock, rp->ai_addr, rp->ai_addrlen) != -1){
+			break;
+		}
+
+		return -1;
+	}
+	if(!rp){
 		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", strerror(errno));
 		return -1;
 	}
+	freeaddrinfo(ainfo);
 
-	memset(&addr, 0, sizeof(struct sockaddr_in));
-	addr.sin_family = AF_INET;
-	addr.sin_port = htons(port);
-	inet_aton(ip, &(addr.sin_addr));
-
-	if(connect(sock, (struct sockaddr*)&addr, sizeof(struct sockaddr_in)) == -1){
-		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", strerror(errno));
+	/* Set non-blocking */
+	opt = fcntl(sock, F_GETFL, 0);
+	if(opt == -1 || fcntl(sock, F_SETFL, opt | O_NONBLOCK) == -1){
+		close(sock);
 		return -1;
 	}
 
@@ -288,6 +311,10 @@ int mqtt3_net_read(mqtt3_context *context)
 		if(read_length == 1){
 			bytes_received++;
 			context->in_packet.command = byte;
+#ifdef WITH_BROKER
+			/* Clients must send CONNECT as their first command. */
+			if(context->connected == false && (byte&0xF0) != CONNECT) return 1;
+#endif
 		}else{
 			if(read_length == 0) return 1; /* EOF */
 			if(errno == EAGAIN || errno == EWOULDBLOCK){
@@ -377,6 +404,9 @@ int mqtt3_net_write(mqtt3_context *context)
 			write_length = write(context->sock, &packet->command, 1);
 			if(write_length == 1){
 				bytes_sent++;
+#ifdef WITH_CLIENT
+				packet->command_saved = packet->command;
+#endif
 				packet->command = 0;
 			}else{
 				if(write_length == 0) return 1; /* EOF */
@@ -434,6 +464,11 @@ int mqtt3_net_write(mqtt3_context *context)
 
 		msgs_sent++;
 
+#ifdef WITH_CLIENT
+		if(client_net_write_callback){
+			client_net_write_callback(packet->command_saved&0xF0);
+		}
+#endif
 		/* Free data and reset values */
 		context->out_packet = packet->next;
 		mqtt3_context_packet_cleanup(packet);
